#include <amsmf/InputStream.h>
#include <amsmf/Exception.h>
#include <stdio.h>

namespace AMSMF
{

class InputStream::Impl_
{
	static const size_t BUFSIZE=1024;
	FILE *m_fp;
	U8 m_buffer[BUFSIZE];
public:
	Impl_(const char *file) : m_fp(fopen(file, "rb"))
	{
		if (!m_fp)
			throw Ex::CannotOpenFile();
	}
	~Impl_()
	{
		if (m_fp)
			fclose(m_fp);
	}
	void read(InputBuffer &ib)
	{
		size_t len;

		len = fread(m_buffer, 1, BUFSIZE, m_fp);
		if (len == 0) {
			if (ferror(m_fp))
				throw Ex::CannotReadFile();
			ib = InputBuffer();
		} else
			ib = InputBuffer(m_buffer, (U32)len);
	}
};

InputStream::InputStream(const char *file) : m_impl(new Impl_(file)) { }

InputStream::~InputStream() { }

void
InputStream::read_(InputBuffer &ib)
{
	if (m_impl.is_valid())
		m_impl->read(ib);
}

void
InputStream::reset(const char *file)
{
	this->clear_buffer();
	m_impl.reset();
	m_impl.reset(new Impl_(file));
}

} // AMSMF
