#ifndef AMSMF_ELEMENTS_H_
#define AMSMF_ELEMENTS_H_

#include <amt/Defines.h>
#include <amsmf/Types.h>
#include <amsmf/Assert.h>
#include <amsmf/SMPTE.h>
#include <amsmf/Encoding.h>

namespace AMSMF
{

class SMFHeader
{
	U16 m_format;
	U16 m_num_tracks;
	U16 m_division;
	U32 m_total_header_size;
	U32 m_num_extra_bytes;
public:
	AMT_ACSOR_RW(U16, format);
	AMT_ACSOR_RW(U16, num_tracks);
	AMT_ACSOR_RW(U16, division);
	AMT_ACSOR_RW(U32, total_header_size);
	AMT_ACSOR_RW(U32, num_extra_bytes);
};

class DivClock
{
public:
	static bool is_match(const SMFHeader &hdr)
	{
		return !FMT::SMF::TimeBase::is_realtime(hdr.division());
	}
private:
	U16 m_num_clocks;
	static U16 tocpb_(const SMFHeader &hdr)
	{
		AMSMF_ASSERT_U_("tocpb_", is_match(hdr));
		return FMT::SMF::TimeBase::get_num_clocks(hdr.division());
	}
public:
	DivClock() { }
	DivClock(const SMFHeader &hdr) : m_num_clocks(tocpb_(hdr)) { }
	void reset(const SMFHeader &hdr) { m_num_clocks = tocpb_(hdr); }
	AMT_ACSOR_R(U16, num_clocks);
};

class DivRealtime
{
public:
	static bool is_match(const SMFHeader &hdr)
	{
		return FMT::SMF::TimeBase::is_realtime(hdr.division());
	}
private:
	U16 m_fps;
	SMPTE::FPSType m_type;
	U16 m_num_subframes;
	static U16 tofps_(const SMFHeader &hdr)
	{
		AMSMF_ASSERT_U_("tofps_", this->is_match(hdr));
		return FMT::SMF::TimeBase::get_num_fps(hdr.division());
	}
	static U16 tosf_(const SMFHeader &hdr)
	{
		return FMT::SMF::TimeBase::get_num_subframes(hdr.division());
	}
	static SMPTE::FPSType lookup_fpstype_(U16);
public:
	DivRealtime() { }
	DivRealtime(const SMFHeader &hdr)
		: m_fps(tofps_(hdr)),
		  m_type(lookup_fpstype_(m_fps)),
		  m_num_subframes(tosf_(hdr))
	{ }
	void reset(const SMFHeader &hdr)
	{
		m_fps = this->tofps_(hdr);
		m_type = this->lookup_fpstype_(m_fps);
		m_num_subframes = this->tosf_(hdr);
	}
	AMT_ACSOR_R(U16, fps);
	AMT_ACSOR_R(SMPTE::FPSType, type);
	AMT_ACSOR_R(U16, num_subframes);
};

}

#endif
