#include <amsmf/ParseSMF.h>
#include <amsmf/InputStream.h>
#include <stdio.h>

using namespace AMT;
using namespace AMSMF;
using namespace AMSMF::Parsers;

class TestHooks : public Hooks
{
public:
	~TestHooks() { }
	void begin_smf(const SMFHeader &hdr)
	{
		printf("begin_smf\n");
		printf("  format=%u\n", (unsigned)hdr.format());
		printf("  num tracks=%u\n", (unsigned)hdr.num_tracks());
		printf("  division=");
		if (DivClock::is_match(hdr)) {
			DivClock c(hdr);
			printf("%u per beat\n", (unsigned)c.num_clocks());
		} else if (DivRealtime::is_match(hdr)) {
			DivRealtime r(hdr);
			printf("%ufps/%unsf\n",
			       (unsigned)r.fps(), (unsigned)r.num_subframes());
		} else
			throw AMT::Ex::Failure();
		printf("  total header size = %u bytes\n",
		       (unsigned)hdr.total_header_size());
		printf("  header extra bytes = %u bytes\n",
		       (unsigned)hdr.num_extra_bytes());
	}
	void end_smf()
	{
		printf("end_smf\n");
	}
	void begin_header_extra_bytes(U32 len)
	{
		printf("begin_header_extra_bytes: %u bytes\n",
		       (unsigned)len);
	}
	void end_header_extra_bytes()
	{
		printf("end_header_extra_bytes\n");
	}
	void header_extra_bytes(const U8 *, U32)
	{
		printf("header_extra_bytes\n");
	}
	void begin_track(int no, U32 len)
	{
		printf("begin_track: #%d, len=%u\n", no, (unsigned)len);
	}
	void end_track(int no)
	{
		printf("end_track: #%d\n", no);
	}
	void track_event(int tr, const TrackEvent &ev)
	{
		printf("  track_event delta=%d ", ev.delta());
		if (Parsers::ChNoteOn::is_match(ev)) {
			Parsers::ChNoteOn rd(ev);
			printf("note on : ch=%d, key=%d, velo=%d\n",
			       rd.channel(), rd.key_number(), rd.velocity());
		} else if (Parsers::ChNoteOff::is_match(ev)) {
			Parsers::ChNoteOff rd(ev);
			printf("note off : ch=%d, key=%d, velo=%d\n",
			       rd.channel(), rd.key_number(), rd.velocity());
		} else if (Parsers::ChPolyKeyPressure::is_match(ev)) {
			Parsers::ChPolyKeyPressure rd(ev);
			printf("poly key pressure : ch=%d, key=%d, int=%d\n",
			       rd.channel(), rd.key_number(), rd.intensity());
		} else if (Parsers::ChControlChange::is_match(ev)) {
			Parsers::ChControlChange rd(ev);
			printf("ctrl change : ch=%d, no=%d, val=%d\n",
			       rd.channel(), rd.number(), rd.value());
		} else if (Parsers::ChProgramChange::is_match(ev)) {
			Parsers::ChProgramChange rd(ev);
			printf("prog change : ch=%d, no=%d\n",
			       rd.channel(), rd.number());
		} else if (Parsers::ChChannelPressure::is_match(ev)) {
			Parsers::ChChannelPressure rd(ev);
			printf("ch pressure : ch=%d, int=%d\n",
			       rd.channel(), rd.intensity());
		} else if (Parsers::ChPitchBend::is_match(ev)) {
			Parsers::ChPitchBend rd(ev);
			printf("bend : ch=%d, val=%d\n",
			       rd.channel(), rd.value());
		} else if (Parsers::SMFMetaSeqNo::is_match(ev)) {
			Parsers::SMFMetaSeqNo rd(ev);
			printf("meta seqno : no=%d\n", rd.number());
		} else if (Parsers::SMFMetaText::is_match(ev)) {
			Parsers::SMFMetaText rd(ev);
			printf("meta text : \"%s\"\n", rd.text());
		} else if (Parsers::SMFMetaCopyright::is_match(ev)) {
			Parsers::SMFMetaCopyright rd(ev);
			printf("meta copyright : \"%s\"\n", rd.text());
		} else if (Parsers::SMFMetaSeqTrackName::is_match(ev)) {
			Parsers::SMFMetaSeqTrackName rd(ev);
			printf("meta seq/track name : \"%s\"\n", rd.text());
		} else if (Parsers::SMFMetaInstName::is_match(ev)) {
			Parsers::SMFMetaInstName rd(ev);
			printf("meta inst name : \"%s\"\n", rd.text());
		} else if (Parsers::SMFMetaLyric::is_match(ev)) {
			Parsers::SMFMetaLyric rd(ev);
			printf("meta lyric : \"%s\"\n", rd.text());
		} else if (Parsers::SMFMetaMarker::is_match(ev)) {
			Parsers::SMFMetaMarker rd(ev);
			printf("meta marker : \"%s\"\n", rd.text());
		} else if (Parsers::SMFMetaQuePoint::is_match(ev)) {
			Parsers::SMFMetaQuePoint rd(ev);
			printf("meta que point : \"%s\"\n", rd.text());
		} else if (Parsers::SMFMetaChPrefix::is_match(ev)) {
			Parsers::SMFMetaChPrefix rd(ev);
			printf("meta ch prefix: \"%d\"\n", rd.channel());
		} else if (Parsers::SMFMetaEndOfTrack::is_match(ev)) {
			Parsers::SMFMetaEndOfTrack rd(ev);
			printf("meta end of track\n");
		} else if (Parsers::SMFMetaSetTempo::is_match(ev)) {
			Parsers::SMFMetaSetTempo rd(ev);
			printf("meta set tempo : %dusec per quarter\n",
			       rd.tempo());
		} else if (Parsers::SMFMetaSMPTEOffset::is_match(ev)) {
			Parsers::SMFMetaSMPTEOffset rd(ev);
			printf("meta : %02d:%02d:%02d.%02d/%02d\n",
			       rd.hour(), rd.minute(), rd.second(),
			       rd.frame(), rd.subframe());
		} else if (Parsers::SMFMetaTimeSign::is_match(ev)) {
			Parsers::SMFMetaTimeSign rd(ev);
			printf("meta time sign : "
			       "%d/%d, %d midi-clocks/click, "
			       "%d 32rd-note/quater\n",
			       rd.numerator(), rd.calc_denominator(),
			       rd.num_clocks_per_click(),
			       rd.num_32nd_notes_per_quater());
		} else if (Parsers::SMFMetaKeySign::is_match(ev)) {
			Parsers::SMFMetaKeySign rd(ev);
			printf("meta key sign : %s %s\n",
			       rd.key_sign_name(), rd.major_minor_name());
		} else if (Parsers::SMFSysEx::is_match(ev)) {
			Parsers::SMFSysEx rd(ev);
			printf("sysex : len=%d, ", rd.length());
			for (U32 i=0; i<rd.length(); i++)
				printf("%02X ", rd.bytes()[i]);
			printf("\n");
		} else if (Parsers::SMFArbitBytes::is_match(ev)) {
			Parsers::SMFArbitBytes rd(ev);
			printf("arbitrary bytes : ");
			for (U32 i=0; i<rd.length(); i++)
				printf("%02X ", rd.bytes()[i]);
			printf("\n");
		} else {
			printf("unknown event(%s: 0x%02X).\n",
			       ev.type_name(),
			       ev.bytes()[ev.is_midi() ? 0 : 1]);
		}
	}
};

int
main(int argc, char **argv)
{
	if (argc != 2) {
		fprintf(stderr, "usage: test_ParseSMF000 <file>\n");
		return 1;
	}
	InputStream s(argv[1]);
	TestHooks h;
	Parser p(h);
	while (p.parse(s.read()))
		;
	return 0;
}
