#include <mseq/SortedDoubleLink.h>

#ifdef TEST_SortedDoubleLink
#include <iostream>

using namespace std;
using namespace MSeq::SortedDoubleLink;


class Test
{
public:
	class Access;
	friend class Test::Access;
	typedef int Ordinal;
private:
	static int s_id;
	int m_id;
	int m_value;
	Node<Test> m_node;
public:
	Test(int n) : m_id(++s_id), m_value(n) { }
	int to_i() const { return m_value; }
	int to_id() const { return m_id; }
};
int Test::s_id = 0;

ostream &
operator << (ostream &os, const Test &t)
{
	os << t.to_id() << ":" << t.to_i();
}

class Test::Access
{
	friend class Container<Test, Test::Access>;
	static Node<Test> *get_node(Test *p)
	{
		return &p->m_node;
	}
	static void dispose_one_(Test *n)
	{
		delete n;
	}
	static void dispose(Container<Test, Test::Access> *c)
	{
		c->remove_all(&dispose_one_);
	}
	static int compare(const Test &p, const Test &q)
	{
		return p.to_i() - q.to_i();
	}
	static int compare(const Test &p, int i)
	{
		return p.to_i() - i;
	}
};

class Dump
{
public:
	Dump() { }
	bool operator () (const Test *t) const
	{
		cout << *t << " ";
		return false;
	}
};

void
dump(const Container<Test, Test::Access> &c)
{
	c.each(Dump());
	cout << endl;
}

int
main()
{
	Container<Test, Test::Access> c;
	Test *t1, *t2, *t3, *tmp;

	c.insert_tail(t1 = new Test(10));
	c.insert_tail(t2 = new Test(20));
	c.insert_tail(t3 = new Test(30));
	c.insert_tail(new Test(30));
	try {
		c.insert_tail(tmp = new Test(20));
	}
	catch (MSeq::Assert::Failure &) {
		cerr << "(expected assertion)" << endl;
		delete tmp;
	}

	try {
		c.insert_before(NULL, tmp = new Test(20));
	}
	catch (MSeq::Assert::Failure &) {
		cerr << "(expected assertion)" << endl;
		delete tmp;
	}
	dump(c);
	tmp = c.find_after(15);
	cout << "find_after(15): ";
	if (tmp) cout << *tmp; else cout << "NULL";
	cout << endl;
	tmp = c.find_before(15);
	cout << "find_before(15): ";
	if (tmp) cout << *tmp; else cout << "NULL";
	cout << endl;
	tmp = c.find_after(30);
	cout << "find_after(30): ";
	if (tmp) cout << *tmp; else cout << "NULL";
	cout << endl;
	tmp = c.find_before(30);
	cout << "find_before(30): ";
	if (tmp) cout << *tmp; else cout << "NULL";
	cout << endl;
	c.insert_suitable_first(new Test(30));
	c.insert_suitable_first(new Test(40));
	dump(c);
	c.insert_suitable_last(new Test(30));
	c.insert_suitable_last(new Test(50));
	dump(c);
	c.insert_suitable_last(c.remove(t3));
	dump(c);
	c.insert_suitable_first(c.remove(t3));
	dump(c);
}
#endif
