#ifndef AMT_DEFINES_H_
#define AMT_DEFINES_H_

#include <amt/Config.h>

#include <cstddef>

#define AMT_NOCOPY(n)							      \
private:								      \
	n(const n &);							      \
	n & operator = (const n &)

#define AMT_DECL_INNER_FRIEND_CLASS(name)				      \
	class name;							      \
	friend class name

#define AMT_BEGIN_CONTRACT(name)	static void contract_##name##_()
#define AMT_CONFIRM_CONTRACT(name)	((void)&contract_##name##_)

#define AMT_UNUSED	__attribute__ ((unused))

#define AMT_EMBED_IMPL							      \
private:								      \
	AMT_DECL_INNER_FRIEND_CLASS(Impl_);				      \
	Impl_ &impl_() const { return *m_impl; }			      \
	AMT::Ptr::Scoped<Impl_> m_impl

#define AMT_DEF_INLINE_DTOR(name)					      \
inline name::~name() { } class name

#define AMT_ACSOR_R(type, elem)						      \
type const &elem() const { return m_##elem; }
#define AMT_ACSOR_W(type, elem)						      \
type &elem() { return m_##elem; }
#define AMT_ACSOR_RW(type, elem)					      \
AMT_ACSOR_R(type, elem)							      \
AMT_ACSOR_W(type, elem)

#define AMT_MUTABLE_ACSOR(type, elem)					      \
type &elem() const { return m_##elem; }

#define AMT_PRIV_ACSOR_R(type, elem)					      \
type const &elem##_() const { return m_##elem; }
#define AMT_PRIV_ACSOR_W(type, elem)					      \
type &elem##_() { return m_##elem; }
#define AMT_PRIV_ACSOR_RW(type, elem)					      \
AMT_PRIV_ACSOR_R(type, elem)						      \
AMT_PRIV_ACSOR_W(type, elem)

#define AMT_PRIV_MUTABLE_ACSOR(type, elem)				      \
type &elem##_() const { return m_##elem; }

#define AMT_INDIRECT_ACSOR_R(type, func, elem)				      \
type const &elem() const { return this->func(); }
#define AMT_INDIRECT_ACSOR_W(type, func, elem)				      \
type &elem() { return this->func(); }
#define AMT_INDIRECT_ACSOR_RW(type, func, elem)				      \
AMT_INDIRECT_ACSOR_R(type, func, elem)					      \
AMT_INDIRECT_ACSOR_W(type, func, elem)

#define AMT_PROXY_ACSOR_R(type, acsor, elem, subacsor)			      \
type const &acsor() const { return m_##elem.subacsor(); }
#define AMT_PROXY_ACSOR_W(type, acsor, elem, subacsor)			      \
type &acsor() { return m_##elem.subacsor(); }
#define AMT_PROXY_ACSOR_RW(type, acsor, elem, subacsor)			      \
AMT_PROXY_ACSOR_R(type, acsor, elem, subacsor)				      \
AMT_PROXY_ACSOR_W(type, acsor, elem, subacsor)

#define AMT_PTR_TO_REF_ACSOR_CONST(type, elem)				      \
type &elem() const { return *m_##elem; }
#define AMT_PTR_TO_REF_ACSOR(type, elem)				      \
type &elem() { return *m_##elem; }

#define AMT_REF_ACSOR_CONST(type, elem)					      \
type &elem() const { return m_##elem; }
#define AMT_REF_ACSOR(type, elem)					      \
type &elem() { return m_##elem; }

#define AMT_DECL_ACSOR_R(type, elem)	type const &elem() const
#define AMT_DECL_ACSOR_W(type, elem)	type &elem()
#define AMT_DECL_ACSOR_RW(type, elem)					      \
AMT_DECL_ACSOR_R(type, elem);						      \
AMT_DECL_ACSOR_W(type, elem)

#define AMT_DEF_IMPL_ACSOR_R(cls, type, elem)				      \
type const &cls::elem() const						      \
{ return const_cast<const Impl_ &>(*m_impl).elem(); }
#define AMT_DEF_IMPL_ACSOR_W(cls, type, elem)				      \
type &cls::elem() { return m_impl->elem(); }
#define AMT_DEF_IMPL_ACSOR_RW(cls, type, elem)				      \
AMT_DEF_IMPL_ACSOR_R(cls, type, elem);					      \
AMT_DEF_IMPL_ACSOR_W(cls, type, elem)

namespace AMT
{

template <typename T_> void swap(T_ *a, T_ *b)
{ T_ tmp = *b; *b = *a; *a = tmp; }

template <typename T_> T_ renounce(T_ *ptr, T_ v=T_())
{ swap(ptr, &v); return v; }

template <typename T_>
struct Const
{
	typedef T_ Deconst;
	typedef const T_ Enconst;
};
template <typename T_>
struct Const<const T_>
{
	typedef T_ Deconst;
	typedef const T_ Enconst;
};



}

#endif
