#ifndef AMT_ENDIAN_H_
#define AMT_ENDIAN_H_

#include <amt/Types.h>
#include <cstring>

namespace AMT
{

namespace Endian
{

struct HostToExt { };
struct ExtToHost { };

template <const Size ofs>
struct Offset
{
	static const Size OFFSET = ofs;
	static const Size LENGTH = 0;
};

template <typename T, class Prev = Offset<0> >
class Field
{
public:
	typedef T ValueType;
	static const Size OFFSET = Prev::OFFSET + Prev::LENGTH;
	static const Size LENGTH = sizeof (T);
private:
	ValueType m_value;
public:
	Field() { }
	Field(ValueType v) : m_value(v) { }
	AMT_ACSOR_RW(ValueType, value);
	static Size offset() { return OFFSET; }
	static Size length() { return LENGTH; }
};

template <class Prev>
class Total
{
public:
	static const Size OFFSET = 0;
	static const Size LENGTH = Prev::OFFSET + Prev::LENGTH;
};

template <class Inh_>
struct Base_
{
	struct To
	{
		template <typename Int_>
		static void convert(const Int_ *v, void *ptr, Size ofs = 0)
		{
			Int_ cv=Inh_::template to<Int_>(*v);
			std::memcpy((char *)ptr + ofs, &cv, sizeof (cv));
		}
		static void convert(const U8 *v, void *ptr, Size ofs = 0)
		{
			*((char *)ptr + ofs) = *v;
		}
		template <typename Int_>
		static Int_ convert(Int_ v)
		{
			return Inh_::template to<Int_>(v);
		}
		template <typename Int_, class Prev_>
		static void convert(const Field<Int_, Prev_> *v, void *ptr)
		{
			To::template convert<Int_>(
				&v->value(), ptr, v->offset());
		}
		template <class Field_>
		static void store(void *ptr, typename Field_::ValueType v)
		{
			typedef typename Field_::ValueType Int_;
			To::template convert<Int_>(&v, ptr, Field_::OFFSET);
		}
	};
	struct From
	{
		template <typename Int_>
		static void convert(Int_ *v, const void *ptr, Size ofs=0)
		{
			std::memcpy(v, (const char *)ptr + ofs, sizeof (*v));
			*v = Inh_::template from<Int_>(*v);
		}
		static void convert(U8 *v, const void *ptr, Size ofs = 0)
		{
			*v = *((const char *)ptr + ofs);
		}
		template <typename Int_>
		static Int_ convert(Int_ v)
		{
			return Inh_::template from<Int_>(*v);
		}
		template <typename Int_, class Prev_>
		static void convert(Field<Int_, Prev_> *v, const void *ptr)
		{
			From::template convert<Int_>(
				&v->value(), ptr, v->offset());
		}
		template <class Field_>
		static typename Field_::ValueType fetch(const void *ptr)
		{
			typedef typename Field_::ValueType Int_;
			Int_ v;
			From::template convert<Int_>(&v, ptr, Field_::OFFSET);
			return v;
		}
	};
};

struct Big : public Base_<Big>
{
	template <typename Int_> static Int_ to(Int_);
	template <typename Int_> static Int_ from(Int_);
	template <typename> struct Convert;
};

template <> struct Big::Convert<HostToExt> : public Big::To { };
template <> struct Big::Convert<ExtToHost> : public Big::From { };

struct Little : public Base_<Little>
{
	template <typename Int_> static Int_ to(Int_);
	template <typename Int_> static Int_ from(Int_);
	template <typename> struct Convert;
};

template <> struct Little::Convert<HostToExt> : public Little::To { };
template <> struct Little::Convert<ExtToHost> : public Little::From { };

}

}

#ifndef AMT_ENDIAN_HEADER_
#error macro AMT_ENDIAN_HEADER_ is not defined.
#endif
#include AMT_ENDIAN_HEADER_

#endif
