/*-
 * Copyright (c)2007 Takuya SHIOZAKI,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef AMT_POINTER_H_
#define AMT_POINTER_H_

#include <amt/Defines.h>

namespace AMT
{

namespace Ptr
{

struct SimpleDisposer
{
	template <typename T_>
	static void dispose(T_ *ptr) { delete ptr; }
};

template <typename D_ = SimpleDisposer>
struct DisposerFunctor
{
	template <typename T_>
	void operator () (T_ *ptr) const
	{
		D_::dispose(ptr);
	}
};

template <typename D_ = SimpleDisposer>
struct PairedFirstDisposer
{
	template <typename Pair_>
	void operator () (Pair_ &p) const
	{
		D_::dispose(p.first);
	}
};

template <typename D_ = SimpleDisposer>
struct PairedSecondDisposer
{
	template <typename Pair_>
	void operator () (Pair_ &p) const
	{
		D_::dispose(p.second);
	}
};

}

}

#endif
