/*-
 * Copyright (c)2007 Takuya SHIOZAKI,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef AMT_SCOPEDPTR_H_
#define AMT_SCOPEDPTR_H_

#include <amt/Pointer.h>

namespace AMT
{

namespace Ptr
{

template <class I_, class T_, class D_>
class ScopedBase
{
	AMT_NOCOPY(ScopedBase);
private:
	T_ *m_ptr;
protected:
	explicit ScopedBase(T_ *p) : m_ptr(p) { }
	ScopedBase() : m_ptr(NULL) { }
	void dispose_(T_ *p)
	{ D_::dispose(static_cast<I_ &>(*this), p); }
public:
	~ScopedBase() { this->reset(); }
	void reset(T_ *p=NULL)
	{ AMT::swap(&m_ptr, &p); if (p) this->dispose_(p); }
	T_ *renounce() { return AMT::renounce(&m_ptr, (T_ *)NULL); }
	T_ &operator * () const { return *m_ptr; }
	T_ *operator -> () const { return m_ptr; }
	T_ &operator [] (int idx) const { return m_ptr[idx]; }
	bool is_valid() const { return m_ptr != NULL; }
};

namespace Bits_
{

template <class I_, class T_, class D_>
struct ScopedDisposer { static void dispose(I_ &, T_ *p) { D_::dispose(p); } };

}

template <class T_, class D_ = SimpleDisposer>
class Scoped : public ScopedBase<Scoped<T_, D_>, T_,
				 Bits_::ScopedDisposer<Scoped<T_, D_>, T_,
						       D_> >
{
	friend class ScopedBase<Scoped<T_, D_>, T_,
				Bits_::ScopedDisposer<Scoped<T_, D_>, T_,
						      D_> >;
	typedef ScopedBase<Scoped<T_, D_>, T_,
			   Bits_::ScopedDisposer<Scoped<T_, D_>, T_,
						 D_> > Super_;
public:
	Scoped() : Super_() { }
	Scoped(T_ *p) : Super_(p) { }
};

}

}

#endif
