#ifndef LIBVSQ_CLOCK_LIST_H_
#define LIBVSQ_CLOCK_LIST_H_

#include <libvsq/Defines.h>
#include <libvsq/Types.h>
#include <libvsq/Assert.h>
#include <amt/ScopedPtr.h>

namespace LibVSQ
{

namespace ClockLists
{

template <class Entry_>
class Pair
{
	ClockLength m_pos;
	Entry_ m_entry;
public:
	explicit Pair(ClockLength p=0) : m_pos(p), m_entry() { }
	~Pair() { }
	AMT_ACSOR_R(ClockLength, pos);
	AMT_ACSOR_RW(Entry_, entry);
};

} // ClockLists

template <class Entry_>
class ClockList
{
	AMT_DECL_INNER_FRIEND_CLASS(Impl_);
public:
	typedef Entry_ Entry;
	typedef ClockLists::Pair<Entry_> Pair;
private:
	AMT::Ptr::Scoped<Impl_> m_impl;
	Pair &at_(int) const;
	Pair &at__(int idx) const
	{
		LIBVSQ_ASSERT_U_("operator []",
				 idx >= 0 && idx < this->num_entries());
		return this->at_(idx);
	}
public:
	ClockList();
	~ClockList();
	Entry_ &insert(ClockLength);
	int num_entries() const;
	Pair &operator [] (int idx) { return this->at__(idx); }
	const Pair &operator [] (int idx) const { return this->at__(idx); }
};


} // LibVSQ

#endif
