#include <libvsq/ClockList.h>
#include <vector>
#include <functional>
#include <algorithm>

namespace LibVSQ
{

template <class Entry_>
class ClockList<Entry_>::Impl_
{
public:
	typedef ClockLists::Pair<Entry_> Pair;
private:
	typedef std::vector<Pair> Vector_;
	typedef typename Vector_::iterator Iter_;
	typedef std::reverse_iterator<Iter_> RIter_;
	typedef typename Vector_::const_iterator CIter_;
	Vector_ m_entries;
public:
	Impl_() { }
	~Impl_() { }
	Entry_ &insert(ClockLength pos)
	{
		RIter_ i = m_entries.rbegin();

		while (i != m_entries.rend() && i->pos() > pos)
			++i;

		return m_entries.insert(i.base(), Pair(pos))->entry();
	}
	int num_entries() const { return m_entries.size(); }
	Pair &at(int idx)
	{
		LIBVSQ_ASSERT_I_("at", idx >= 0 && idx < this->num_entries());
		return m_entries[idx];
	}
};

template <class Entry_>
ClockList<Entry_>::ClockList() : m_impl(new Impl_) { }

template <class Entry_>
ClockList<Entry_>::~ClockList() { }

template <class Entry_>
Entry_ &ClockList<Entry_>::insert(ClockLength pos)
{ return m_impl->insert(pos); }

template <class Entry_>
int ClockList<Entry_>::num_entries() const { return m_impl->num_entries(); }

template <class Entry_>
typename ClockList<Entry_>::Pair &ClockList<Entry_>::at_(int idx) const
{ return m_impl->at(idx); }

} // LibVSQ
