#ifndef LIBVSQ_DATABASE_TCC_
#define LIBVSQ_DATABASE_TCC_

#include <libvsq/Database.h>
#include <libvsq/Exception.h>
#include <vector>
#include <map>

namespace LibVSQ
{

template <class Data_>
class Database<Data_>::Impl_
{
	typedef std::map<std::string, int> Map_;
	typedef std::vector<Data_ *> Vector_;
	typedef AMT::Ptr::DisposerFunctor<> D_;
	typedef typename Map_::value_type Pair_;
	Vector_ m_datas;
	Map_ m_map;
public:
	Impl_() { }
	~Impl_() { std::for_each(m_datas.begin(), m_datas.end(), D_()); }
	void insert(const std::string &n, Data_ *d)
	{
		AMT::Ptr::Scoped<Data_> data(d); // for exception safe.
		if (m_map.find(n) != m_map.end())
			throw Ex::DuplicatedEntry();
		m_map.insert(Pair_(n, m_datas.size()));
		m_datas.push_back(d);
		data.renounce();
	}
	int num_entries() const { return m_datas.size(); }
	Data_ &at_(int idx) { return *m_datas[idx]; }
	int find_index(const std::string &n) const
	{
		typename Map_::const_iterator i = m_map.find(n);

		if (i==m_map.end())
			throw Ex::NoSuchEntry();

		return i->second;
	}
	Data_ &find(const std::string &n)
	{
		return *m_datas[this->find_index(n)];
	}
};

template <class Data_>
Database<Data_>::Database() : m_impl(new Impl_) { }

template <class Data_>
Database<Data_>::~Database() { }

template <class Data_>
void Database<Data_>::insert(const std::string &n, Data_ *d)
{ m_impl->insert(n, d); }

template <class Data_>
int Database<Data_>::find_index(const std::string &n) const
{ return m_impl->find_index(n); }

template <class Data_>
int Database<Data_>::num_entries() const { return m_impl->num_entries(); }

template <class Data_>
Data_ &Database<Data_>::at_(int idx) const { return m_impl->at_(idx); }

template <class Data_>
Data_ &Database<Data_>::find(const std::string &n) { return m_impl->find(n); }

template <class Data_>
const Data_ &Database<Data_>::find(const std::string &n) const
{ return m_impl->find(n); }

} // LibVSQ

#endif
