#ifndef LIBVSQ_EVENT_DATA_H_
#define LIBVSQ_EVENT_DATA_H_

#include <libvsq/Defines.h>
#include <libvsq/Types.h>
#include <libvsq/Assert.h>
#include <libvsq/Exception.h>
#include <string>

namespace LibVSQ
{

class EventHandle;

class EventData
{
public:
	enum Type
	{
		T_SINGER,
		T_ANOTE,
		T_EOS
	};
private:
	Type m_type;
public:
	virtual ~EventData() = 0;
	EventData(Type t) : m_type(t) { }
	AMT_ACSOR_R(Type, type);
	template <class Type_>
	Type_ &to()
	{
		if (this->type() != Type_::TYPE)
			throw Ex::UnmatchedType();
		return *static_cast<Type_ *>(this);
	}
	template <class Type_>
	const Type_ &to() const
	{
		if (this->type() != Type_::TYPE)
			throw Ex::UnmatchedType();
		return *static_cast<const Type_ *>(this);
	}
};
AMT_DEF_INLINE_DTOR(EventData);

namespace EventDatas
{

class Singer : public EventData
{
	EventHandle *m_icon_handle;
public:
	static const Type TYPE = T_SINGER;
	~Singer() { }
	Singer() : EventData(TYPE), m_icon_handle(NULL) { }
	AMT_PTR_TO_REF_ACSOR_CONST(const EventHandle, icon_handle);
	AMT_ACSOR_W(EventHandle *, icon_handle);
};

class ANote : public EventData
{
	ClockLength m_length;
	U8 m_note_no;
	U8 m_dynamics;
	U8 m_pm_bend_depth;
	U8 m_pm_bend_length;
	U8 m_portament_use;
	U8 m_decay_rate;
	U8 m_accent;
	EventHandle *m_lyric_handle;
	EventHandle *m_vibrato_handle;
	ClockLength m_vibrato_delay;
public:
	static const Type TYPE = T_ANOTE;
	ANote() : EventData(TYPE), m_length(0), m_note_no(0), m_dynamics(0),
		  m_pm_bend_depth(0), m_pm_bend_length(0), m_portament_use(0),
		  m_decay_rate(0), m_accent(0), m_lyric_handle(NULL),
		  m_vibrato_handle(NULL), m_vibrato_delay(0)
	{ }
	~ANote() { }
	AMT_ACSOR_RW(ClockLength, length);
	AMT_ACSOR_RW(U8, note_no);
	AMT_ACSOR_RW(U8, dynamics);
	AMT_ACSOR_RW(U8, pm_bend_depth);
	AMT_ACSOR_RW(U8, pm_bend_length);
	AMT_ACSOR_RW(U8, portament_use);
	AMT_ACSOR_RW(U8, decay_rate);
	AMT_ACSOR_RW(U8, accent);
	AMT_PTR_TO_REF_ACSOR_CONST(const EventHandle, lyric_handle);
	AMT_ACSOR_W(EventHandle *, lyric_handle);
	bool is_vibrato_valid() const { return m_vibrato_handle != NULL; }
	AMT_PTR_TO_REF_ACSOR_CONST(const EventHandle, vibrato_handle);
	AMT_ACSOR_W(EventHandle *, vibrato_handle);
	AMT_ACSOR_RW(ClockLength, vibrato_delay);
};

class EndOfSong : public EventData
{
public:
	static const Type TYPE = T_EOS;
	EndOfSong() : EventData(TYPE) { }
	~EndOfSong() { }
};

} // EventDatas

} // LibVSQ

#endif
