#ifndef LIBVSQ_KEY_DISPATCH_TCC_
#define LIBVSQ_KEY_DISPATCH_TCC_

#include <libvsq/KeyDispatch.h>
#include <libvsq/Exception.h>
#include <utility>
#include <functional>
#include <map>
#include <string.h>

namespace LibVSQ
{

template <class Handler_>
class KeyDispatcher<Handler_>::Impl_
{
public:
	typedef Handler_ Handler;
	typedef typename Handler::Context Context;
	typedef typename Handler::Key Key;
	typedef typename Handler::Arg Arg;
private:
	struct Cmp_ : std::binary_function<Key, Key, bool>
	{
		bool operator () (Key k1, Key k2) const
		{
			return strcmp(k1, k2) < 0;
		}
	};
	typedef std::map<Key, const Handler *, Cmp_> Map_;
	Map_ m_handlers;
public:
	Impl_() { }
	~Impl_() { }
	void insert(const Handler &h)
	{
		Key k = h.key();
		if (m_handlers.find(k) != m_handlers.end())
			throw Ex::DuplicatedEntry();
		m_handlers.insert(std::make_pair(k, &h));
	}
	void operator () (Context &ctx, Key k, Arg a) const
	{
		typename Map_::const_iterator i = m_handlers.find(k);
		if (i == m_handlers.end())
			throw Ex::NoSuchEntry();
		(*i->second)(ctx, k, a);
	}
};

template <class Handler_> KeyDispatcher<Handler_>::~KeyDispatcher() { }

template <class Handler_>
KeyDispatcher<Handler_>::KeyDispatcher() : m_impl(new Impl_) { }

template <class Handler_>
void KeyDispatcher<Handler_>::insert(const Handler_ &h) { m_impl->insert(h); }

template <class Handler_>
void KeyDispatcher<Handler_>::operator () (Context &ctx, Key k, Arg a) const
{ (*m_impl)(ctx, k, a); }

} // LibVSQ

#endif
