#include <libvsq/EventHandle.h>
#include <libvsq/Exception.h>
#include <libvsq/Database.tcc>

namespace LibVSQ
{

namespace EventHandles
{

class VibratoChanges::Impl_
{
	typedef std::vector<VibratoCoord> Vector_;
	int m_num_changes;
	U8 m_start_value;
	Vector_ m_changes;
public:
	Impl_() : m_num_changes(0), m_start_value(0) { }
	~Impl_() { }
	AMT_ACSOR_RW(int, num_changes);
	AMT_ACSOR_RW(U8, start_value);
	VibratoCoord &at_(int idx)
	{
		if (m_changes.size() == 0)
			m_changes.resize(m_num_changes);
		return m_changes[idx];
	}
};

VibratoChanges::VibratoChanges() : m_impl(new Impl_) { }

VibratoChanges::~VibratoChanges() { }

AMT_DEF_IMPL_ACSOR_RW(VibratoChanges, int, num_changes);

AMT_DEF_IMPL_ACSOR_RW(VibratoChanges, U8, start_value);

VibratoCoord &VibratoChanges::at_(int idx) const { return m_impl->at_(idx); }


// ----------------------------------------------------------------------

class Lyric::Impl_
{
	typedef std::vector<LyricField> Vector_;
	Vector_ m_lyrics;
public:
	Impl_() { }
	~Impl_() { }
	int num_lyrics() const { return m_lyrics.size(); }
	LyricField &insert()
	{ m_lyrics.push_back(LyricField()); return m_lyrics.back(); }
	const LyricField &at_(int idx) const
	{
		LIBVSQ_ASSERT_I_("at_", idx < m_lyrics.size());
		return m_lyrics[idx];
	}
};

Lyric::Lyric(const std::string &n) : EventHandle(n, TYPE), m_impl(new Impl_)
{ }

Lyric::~Lyric() { }

int Lyric::num_lyrics() const { return m_impl->num_lyrics(); }

LyricField &Lyric::insert() { return m_impl->insert(); }

const LyricField &Lyric::at_(int idx) const
{ return m_impl->at_(idx); }


} // EventHandles

template class Database<EventHandle>;

} // LibVSQ
