#include <libvsq/LineBuffer.h>
#include <string.h>

namespace LibVSQ
{

namespace
{

inline bool is_newline(char ch) { return ch == '\n'; }
template <typename T_>
inline int cmp(T_ v1, T_ v2) { return v1>v2 ? 1 : v1<v2 ? -1 : 0; }

} // <anon>

LineBuffer::LineBuffer(const char *str)
	: m_buffer(str), m_length(str ? strlen(str) : 0)
{ }

void LineBuffer::chomp()
{
	if (m_length > 0 && is_newline(m_buffer[m_length-1]))
		m_length--;
}

LineBuffer LineBuffer::skip_to(char ch)
{
	const void *ptr;
	if (m_length>0 &&
	    (ptr = memchr(m_buffer, (int)(U8)ch, m_length)) != NULL) {
		LineBuffer ret(m_buffer, (const char *)ptr-m_buffer);
		m_buffer += ret.length();
		m_length -= ret.length();
		return ret;
	}
	return LineBuffer();
}

LineBuffer LineBuffer::separate(char ch)
{
	LineBuffer ret = this->skip_to(ch);
	m_buffer++;
	m_length--;
	return ret;
}

LineBuffer LineBuffer::cut_left(Size len)
{
	if (len >= m_length) {
		LineBuffer ret = *this;
		m_buffer = NULL;
		m_length = 0;
		return ret;
	} else {
		LineBuffer ret(m_buffer, len);
		m_buffer += len;
		m_length -= len;
		return ret;
	}
}

LineBuffer LineBuffer::cut_right(Size len)
{
	if (len >= m_length) {
		LineBuffer ret = *this;
		m_buffer = NULL;
		m_length = 0;
		return ret;
	} else {
		m_length -= len;
		return LineBuffer(m_buffer+m_length, len);
	}
}

int
LineBuffer::compare(const LineBuffer &rh) const
{
	int s = cmp(m_length, rh.length());
	if (!s)
		s = memcmp(m_buffer, rh.buffer(), m_length);
	return s;
}

bool
LineBuffer::match_prefix(const LineBuffer &sub) const
{
	return
	    m_length >= sub.length() &&
	    !memcmp(m_buffer, sub.buffer(), sub.length());
}

bool
LineBuffer::match_suffix(const LineBuffer &sub) const
{
	return
	    m_length >= sub.length() &&
	    !memcmp(m_buffer+m_length-sub.length(),
		    sub.buffer(), sub.length());
}

} // LibVSQ
