#include <libvsq/Mixer.h>
#include <libvsq/Exception.h>
#include <amt/Pointer.h>
#include <vector>
#include <algorithm>

namespace LibVSQ
{

class Mixer::Impl_
{
	static const int NT_UNDEF = -1;
	typedef std::vector<Param> Vector_;
	Param m_master;
	int m_output_mode;
	bool m_is_num_tracks_set;
	Vector_ m_params;
public:
	Impl_() : m_is_num_tracks_set(false) { }
	~Impl_() { }
	AMT_ACSOR_R(bool, is_num_tracks_set);
	int num_tracks() const
	{
		if (!this->is_num_tracks_set())
			throw Ex::NumberUnset();
		return m_params.size();
	}
	void set_num_tracks(int ntr)
	{
		if (this->is_num_tracks_set())
			throw Ex::DuplicatedEntry();
		if (ntr < 0)
			throw Ex::IllegalTrackNumber();
		m_is_num_tracks_set = true;
		if (ntr)
			m_params.resize(ntr);
	}
	AMT_ACSOR_RW(Param, master);
	AMT_ACSOR_RW(int, output_mode);
	Param &at_(int tr)
	{
		if (tr < 0 || tr >= m_params.size())
			throw Ex::IllegalTrackNumber();
		return m_params[tr];
	}
};

Mixer::Mixer() : m_impl(new Impl_) { }

Mixer::~Mixer() { }

int Mixer::num_tracks() const { return m_impl->num_tracks(); }

void Mixer::set_num_tracks(int ntr) { m_impl->set_num_tracks(ntr); }

AMT_DEF_IMPL_ACSOR_RW(Mixer, Mixer::Param, master);

AMT_DEF_IMPL_ACSOR_RW(Mixer, int, output_mode);

Mixer::Param &Mixer::at_(int idx) const { return m_impl->at_(idx); }

} // LibVSQ
