#include <libvsq/ParseVSQText.h>
#include <libvsq/Exception.h>
#include <libvsq/Assert.h>
#include <string>
#include <string.h>

namespace LibVSQ
{

using namespace TextParsers;

namespace
{


const char SECTION_NAME_LEFT='[';
const char SECTION_NAME_RIGHT=']';
const char FIELD_SEP='=';

} // <anon>

class TextParser::Impl_
{
	Hooks &m_hooks;
	std::string m_current_section;
public:
	void done()
	{
		if (!m_current_section.empty()) {
			m_hooks.end_section(m_current_section.c_str());
			std::string().swap(m_current_section);
		}
	}
	void parse(LineBuffer);
	~Impl_() { }
	Impl_(Hooks &h) : m_hooks(h) { }
};

void
TextParser::Impl_::parse(LineBuffer line)
{
	line.chomp();
	if (line[0] == SECTION_NAME_LEFT) {
		// begin new section
		this->done();
		line.cut_left(1);
		LineBuffer name = line.separate(SECTION_NAME_RIGHT);
		if (name.is_empty() || !line.is_empty())
			throw Ex::MalformedSectionName();
		std::string(name.buffer(), name.length()).swap(
			m_current_section);
		m_hooks.begin_section(m_current_section.c_str());
	} else {
		// field
		if (m_current_section.empty())
			throw Ex::SectionIdentifierLacked();
		LineBuffer key = line.separate(FIELD_SEP);
		if (key.is_empty())
			throw Ex::MalformedField();
		m_hooks.field(m_current_section.c_str(),
			      std::string(key.buffer(), key.length()).c_str(),
			      std::string(line.buffer(),
					  line.length()).c_str());
	}
}

TextParser::TextParser(Hooks &h) : m_impl(new Impl_(h)) { }

TextParser::~TextParser() { }

void TextParser::done()
{
	if (m_impl.is_valid())
		m_impl->done();
	m_impl.reset();
}

void TextParser::reset(Hooks &h)
{
	this->done();
	m_impl.reset(new Impl_(h));
}

bool TextParser::parse_(const LineBuffer &l)
{
	if (l.is_empty()) {
		this->done();
		return false;
	}
	m_impl->parse(l);
	return true;
}

} // LibVSQ
