#include <libvsq/ReadSectCommon.h>
#include <libvsq/ReadSection.tcc>
#include <libvsq/CSV.h>
#include <amt/LexCast.h>

namespace LibVSQ
{

namespace
{

#define ALL_SET(x)							      \
	x(T_VERSION)							      \
	x(T_NAME)							      \
	x(T_COLOR)							      \
	x(T_DYNAMICS_MODE)						      \
	x(T_PLAY_MODE)

enum Tag
{
	T_VERSION	= 0x00000001,
	T_NAME		= 0x00000002,
	T_COLOR		= 0x00000004,
	T_DYNAMICS_MODE = 0x00000008,
	T_PLAY_MODE	= 0x00000010,
};

const U32 T_MANDATORY = LIBVSQ_RH_MAKE_MASK(ALL_SET);

class Context : public SimpleContextTmpl<Tracks::Common, Tag, T_MANDATORY>
{
	typedef SimpleContextTmpl<Tracks::Common, Tag, T_MANDATORY> Super_;
public:
	Context(Track &tr, Data &d) : Super_(tr, d) { }
	void check_complete() const
	{ this->Super_::check_complete_mandatory(); }
};

} // <anon>

#define RH_HANDLER(tag)		HandlerTmpl<Context, tag>

LIBVSQ_RH_DEF(T_VERSION, "Version") { ctx.data().version() = a; }

LIBVSQ_RH_DEF(T_NAME, "Name") { ctx.data().name() = a; }

LIBVSQ_RH_DEF(T_COLOR, "Color")
{
	CSV csv(a);
	RGB888 &c = ctx.data().color();

	try {
		c.r = csv.fetch_int();
		c.g = csv.fetch_int();
		c.b = csv.fetch_int();
	}
	catch (Ex::IllegalString &) {
		throw Ex::IllegalRGBString();
	}
	if (!csv.is_end())
		throw Ex::IllegalRGBString();
}

LIBVSQ_RH_DEF(T_DYNAMICS_MODE, "DynamicsMode")
{ ctx.data().dynamics_mode() = (int)AMT::lex_cast<long>(a); }

LIBVSQ_RH_DEF(T_PLAY_MODE, "PlayMode")
{ ctx.data().play_mode() = (int)AMT::lex_cast<long>(a); }

LIBVSQ_RH_REGISTER(Context, ALL_SET);

class ReadSectCommon_ : public ReaderTmpl<Context>
{
	typedef ReaderTmpl<Context> Super_;
public:
	ReadSectCommon_(Track &tr, const char *) : Super_(tr, tr.common()) { }
};

template class ReadSectionFactoryTmpl<ReadSectCommon_>;

} // LibVSQ
