#include <libvsq/ReadSectEventList.h>
#include <libvsq/ReadSection.tcc>
#include <libvsq/Track.h>
#include <libvsq/EventList.h>
#include <libvsq/EventDataDB.h>
#include <libvsq/EventData.h>
#include <amt/LexCast.h>
#include <string.h>

namespace LibVSQ
{

namespace
{
const char EOS[] = "EOS";
}

class ReadSectEventList_ : public ReadSection
{
	EventList &m_events;
	EventDatas::Database &m_db;
	bool m_eos;
public:
	ReadSectEventList_(Track &tr, const char *)
		: m_events(tr.events()), m_db(tr.event_datas()), m_eos(false)
	{ }
	~ReadSectEventList_()
	{
	}
	void operator () (const char *, const char *);
	void check_complete() const
	{
		if (!m_eos)
			throw Ex::UnterminatedEventList();
	}
};

void
ReadSectEventList_::operator () (const char *key, const char *val)
{
	if (m_eos)
		throw Ex::EventListAlreadyTerminated();

	ClockLength pos = (ClockLength)AMT::lex_cast<unsigned long>(key);
	EventDatas::Entry *e = m_events.insert(pos) = &m_db.insert(val);
	if (!strcmp(val, EOS)) {
		m_eos = true;
		e->set_data(new EventDatas::EndOfSong);
	}
}

template class ReadSectionFactoryTmpl<ReadSectEventList_>;

} // LibVSQ
