#include <libvsq/ReadSectID.h>
#include <libvsq/ReadSection.tcc>
#include <libvsq/EventDataDB.h>
#include <libvsq/EventData.h>
#include <libvsq/EventHandle.h>
#include <libvsq/LineBuffer.h>
#include <amt/LexCast.h>

namespace LibVSQ
{

namespace
{

const char FN_TYPE[] = "Type";
const char FT_SINGER[] = "Singer";
const char FT_ANOTE[] = "Anote";

namespace Singer
{

enum Tag
{
	T_ICON_HANDLE=0x00000001,
};

#define SINGER_ALL(x)	x(T_ICON_HANDLE)

const U32 TM_MANDATORY = LIBVSQ_RH_MAKE_MASK(SINGER_ALL);

class Context : public SimpleContextTmpl<EventDatas::Singer, Tag, TM_MANDATORY>
{
	typedef SimpleContextTmpl<EventDatas::Singer, Tag, TM_MANDATORY>Super_;
public:
	Context(Track &tr, Data &d) : Super_(tr, d) { }
	void check_complete() const
	{ this->Super_::check_complete_mandatory(); }
};

class Reader : public ReaderTmpl<Context>
{
	typedef ReaderTmpl<Context> Super_;
public:
	Reader(Track &tr, EventDatas::Entry &e) : Super_(tr, *new Data)
	{
		e.set_data(&m_ctx.data());
	}
};

} // Singer

} // <anon>

#define RH_HANDLER(t)	HandlerTmpl<Singer::Context, Singer::t>

LIBVSQ_RH_DEF(T_ICON_HANDLE, "IconHandle")
{
	ctx.data().icon_handle() =
	    ctx.track().handles().insert<EventHandles::Singer>(a);
}

LIBVSQ_RH_REGISTER(Singer::Context, SINGER_ALL);

#undef RH_HANDLER

namespace
{

namespace Anote
{

enum Tag
{
	T_LENGTH		= 0x00000001,
	T_NOTE_NO		= 0x00000002,
	T_DYNAMICS		= 0x00000004,
	T_PM_BEND_DEPTH		= 0x00000008,
	T_PM_BEND_LENGTH	= 0x00000010,
	T_PORTAMENTO_USE	= 0x00000020,
	T_DECAY_RATE		= 0x00000040,
	T_ACCENT		= 0x00000080,
	T_LYRIC_HANDLE		= 0x00000100,
	T_VIBRATO_HANDLE	= 0x00000200,
	T_VIBRATO_DELAY		= 0x00000400,
};

#define ANOTE_MANDATORY(x)						      \
	x(T_LENGTH)							      \
	x(T_NOTE_NO)							      \
	x(T_DYNAMICS)							      \
	x(T_PM_BEND_DEPTH)						      \
	x(T_PM_BEND_LENGTH)						      \
	x(T_PORTAMENTO_USE)						      \
	x(T_DECAY_RATE)							      \
	x(T_ACCENT)							      \
	x(T_LYRIC_HANDLE)
#define ANOTE_OPT(x)							      \
	x(T_VIBRATO_HANDLE)						      \
	x(T_VIBRATO_DELAY)
#define ANOTE_ALL(x) ANOTE_MANDATORY(x) ANOTE_OPT(x)

const U32 TM_MANDATORY = LIBVSQ_RH_MAKE_MASK(ANOTE_MANDATORY);
const U32 TM_ALL = LIBVSQ_RH_MAKE_MASK(ANOTE_ALL);

class Context : public SimpleContextTmpl<EventDatas::ANote, Tag, TM_MANDATORY>
{
	typedef SimpleContextTmpl<EventDatas::ANote, Tag, TM_MANDATORY> Super_;
public:
	Context(Track &tr, Data &d) : Super_(tr, d) { }
	void check_complete() const
	{
		this->Super_::check_complete_mandatory();
		if (this->is_set(T_VIBRATO_HANDLE, NULL) !=
		    this->is_set(T_VIBRATO_DELAY, NULL))
			throw Ex::LackedEntry();
	}
};

class Reader : public ReaderTmpl<Context>
{
	typedef ReaderTmpl<Context> Super_;
public:
	Reader(Track &tr, EventDatas::Entry &e) : Super_(tr, *new Data)
	{
		e.set_data(&m_ctx.data());
	}
};

} // Anote

} // <anon>

#define RH_HANDLER(t)	HandlerTmpl<Anote::Context, Anote::t>

LIBVSQ_RH_DEF(T_LENGTH, "Length")
{
	ctx.data().length() = (ClockLength)AMT::lex_cast<unsigned long>(a);
}

LIBVSQ_RH_DEF(T_NOTE_NO, "Note#")
{
	ctx.data().note_no() = (U8)AMT::lex_cast<unsigned long>(a);
}

LIBVSQ_RH_DEF(T_DYNAMICS, "Dynamics")
{
	ctx.data().dynamics() = (U8)AMT::lex_cast<unsigned long>(a);
}

LIBVSQ_RH_DEF(T_PM_BEND_DEPTH, "PMBendDepth")
{
	ctx.data().pm_bend_depth() = (U8)AMT::lex_cast<unsigned long>(a);
}

LIBVSQ_RH_DEF(T_PM_BEND_LENGTH, "PMBendLength")
{
	ctx.data().pm_bend_length() = (U8)AMT::lex_cast<unsigned long>(a);
}

LIBVSQ_RH_DEF(T_PORTAMENTO_USE, "PMbPortamentoUse")
{
	ctx.data().portament_use() = (U8)AMT::lex_cast<unsigned long>(a);
}

LIBVSQ_RH_DEF(T_DECAY_RATE, "DEMdecGainRate")
{
	ctx.data().decay_rate() = (U8)AMT::lex_cast<unsigned long>(a);
}

LIBVSQ_RH_DEF(T_ACCENT, "DEMaccent")
{
	ctx.data().accent() = (U8)AMT::lex_cast<unsigned long>(a);
}

LIBVSQ_RH_DEF(T_LYRIC_HANDLE, "LyricHandle")
{
	ctx.data().lyric_handle() =
	    ctx.track().handles().insert<EventHandles::Lyric>(a);
}

LIBVSQ_RH_DEF(T_VIBRATO_HANDLE, "VibratoHandle")
{
	ctx.data().vibrato_handle() =
	    ctx.track().handles().insert<EventHandles::Vibrato>(a);
}

LIBVSQ_RH_DEF(T_VIBRATO_DELAY, "VibratoDelay")
{
	ctx.data().vibrato_delay() =
	    (ClockLength)AMT::lex_cast<unsigned long>(a);
}

LIBVSQ_RH_REGISTER(Anote::Context, ANOTE_ALL);

#undef RH_HANDLER

class ReadSectID_ : public ReadSection
{
	AMT::Ptr::Scoped<ReadSection> m_sub_reader;
	Track &m_track;
	EventDatas::Entry &m_entry;
	static EventDatas::Entry &find_entry_(Track &tr, const char *name)
	{
		return tr.event_datas().find(name);
	}
public:
	ReadSectID_(Track &tr, const char *name)
		: m_track(tr), m_entry(find_entry_(tr, name))
	{ }
	~ReadSectID_() { }
	void operator () (const char *key, const char *val)
	{
		if (m_sub_reader.is_valid()) {
			(*m_sub_reader)(key, val);
			return;
		}
		if (strcmp(key, FN_TYPE))
			throw Ex::TypeEntryLacked();
		else if (!strcmp(val, FT_SINGER))
			m_sub_reader.reset(
				new Singer::Reader(m_track, m_entry));
		else if (!strcmp(val, FT_ANOTE))
			m_sub_reader.reset(
				new Anote::Reader(m_track, m_entry));
		else
			throw Ex::IllegalTypeEntry();
	}
	void check_complete() const
	{
		if (!m_sub_reader.is_valid())
			throw Ex::TypeEntryLacked();
		m_sub_reader->check_complete();
	}
};

template class ReadSectionFactoryTmpl<ReadSectID_>;

} // LibVSQ
