#include <libvsq/ReadSong.h>
#include <libvsq/ParseVSQ.h>
#include <libvsq/ReadTrack.h>
#include <libvsq/Song.h>
#include <libvsq/TempoMap.h>
#include <libvsq/TimeSignMap.h>

namespace LibVSQ
{

class ReadSong::Impl_ : public ParseVSQ::Hooks
{
	AMT::Ptr::Scoped<Song> m_song;
	ParseVSQ m_parse_vsq;
	AMT::Ptr::Scoped<ReadTrack> m_read_track;
	int m_num_voice_tracks;
public:
	Impl_() : m_parse_vsq(*this) { }
	~Impl_() { }
	//
	void begin_vsq(int ntr, ClockLength div)
	{
		m_song.reset(new Song);
		m_num_voice_tracks = ntr;
		m_song->division() = div;
	}
	void end_vsq() { }
	void begin_master_track() { }
	void end_master_track() { }
	void change_tempo(ClockLength pos, double t)
	{
		m_song->tempo_map().insert(pos) = t;
	}
	void change_time_signature(ClockLength pos, int n, int d)
	{
		m_song->time_sign_map().insert(pos).set(n, d);
	}
	void begin_voice_track(int)
	{
		m_read_track.reset(new ReadTrack(m_song->insert_track()));
	}
	void end_voice_track(int)
	{
		m_read_track->check_complete();
		m_read_track.reset();
	}
	void begin_section(int, const char *sect)
	{
		m_read_track->begin_section(sect);
	}
	void end_section(int, const char *)
	{
		m_read_track->end_section();
	}
	void field(int, const char *, const char *key, const char *val)
	{
		m_read_track->field(key, val);
	}
	//
	bool operator () (InputBuffer &ib)
	{
		return m_parse_vsq(ib);
	}
	const Song *done()
	{
		m_parse_vsq.done();
		return m_song.renounce();
	}
};


ReadSong::ReadSong() : m_impl(new Impl_) { }

ReadSong::~ReadSong() { }

bool ReadSong::operator () (InputBuffer &ib)
{
	if (!m_impl.is_valid())
		return false;
	return (*m_impl)(ib);
}

const Song *ReadSong::done()
{
	if (!m_impl.is_valid())
		return NULL;
	const Song *s = m_impl->done();
	m_impl.reset();
	return s;
}

void ReadSong::reset()
{
	delete this->done();
	m_impl.reset(new Impl_);
}

} // LibVSQ
