#include <libvsq/Song.h>
#include <libvsq/Track.h>
#include <libvsq/TempoMap.h>
#include <libvsq/TimeSignMap.h>
#include <vector>

namespace LibVSQ
{

class Song::Impl_
{
	typedef std::vector<Track *> Vector_;
	typedef AMT::Ptr::DisposerFunctor<> D_;
	Vector_ m_tracks;
	TempoMap m_tempo_map;
	TimeSignMap m_time_sign_map;
	ClockLength m_division;
public:
	Impl_() : m_division(0) { }
	~Impl_() { std::for_each(m_tracks.begin(), m_tracks.end(), D_()); }
	AMT_ACSOR_RW(ClockLength, division);
	bool is_master_valid() const
	{
		return m_tracks.size() > 0 && m_tracks[0]->is_master_valid();
	}
	const Tracks::Master &master_() const
	{
		return *m_tracks[0]->master();
	}
	bool is_mixer_valid() const
	{
		return m_tracks.size() > 0 && m_tracks[0]->is_mixer_valid();
	}
	const Mixer &mixer_() const
	{
		return *m_tracks[0]->mixer();
	}
	int num_tracks() const
	{
		return m_tracks.size();
	}
	AMT_ACSOR_RW(TempoMap, tempo_map);
	AMT_ACSOR_RW(TimeSignMap, time_sign_map);
	Track &insert_track()
	{
		AMT::Ptr::Scoped<Track> tr(new Track); // for exception safe

		m_tracks.push_back(NULL);
		m_tracks.back() = &*tr;

		return *tr.renounce();
	}
	Track &track_(int tr) { return *m_tracks[tr]; }
};

Song::Song() : m_impl(new Impl_) { }

Song::~Song() { }

AMT_DEF_IMPL_ACSOR_RW(Song, ClockLength, division);

bool Song::is_master_valid() const { return m_impl->is_master_valid(); }

const Tracks::Master &Song::master_() const { return m_impl->master_(); }

bool Song::is_mixer_valid() const { return m_impl->is_mixer_valid(); }

const Mixer &Song::mixer_() const { return m_impl->mixer_(); }

int Song::num_tracks() const { return m_impl->num_tracks(); }

AMT_DEF_IMPL_ACSOR_RW(Song, TempoMap, tempo_map);

AMT_DEF_IMPL_ACSOR_RW(Song, TimeSignMap, time_sign_map);

Track &Song::insert_track() { return m_impl->insert_track(); }

Track &Song::track_(int tr) const { return m_impl->track_(tr); }

} // LibVSQ
