#include <libvsq/SortDMText.h>
#include <libvsq/Exception.h>
#include <libvsq/TextQueue.h>
#include <libvsq/LineBuffer.h>
#include <amt/LexCast.h>

namespace LibVSQ
{

const char VSQ_TEXT_ID[] = "DM:";
const Size VSQ_TEXT_ID_LEN = sizeof (VSQ_TEXT_ID) -1;
const char VSQ_TEXT_ID_SEP = ':';

class SortDMText::Impl_
{
	int m_initial;
	int m_count;
	TextQueue m_queue;
	static const char *check_vsq_text_(const char *text, int *rnum)
	{
		LineBuffer buf(text);

		if (!buf.match_prefix(VSQ_TEXT_ID))
			return NULL;
		buf.cut_left(VSQ_TEXT_ID_LEN);
		LineBuffer num = buf.skip_to(VSQ_TEXT_ID_SEP);
		if (num.is_empty())
			return NULL;
		buf.cut_left(1);
		try {
			*rnum = (int)AMT::lex_cast<unsigned long>(
				std::string(num.begin(), num.end()), 10);
		}
		catch (Ex::ParseError &) {
			return NULL;
		}

		return buf.buffer();
	}
	void flush_top_()
	{
		if (!m_queue.is_empty()) {
			int tn = m_queue.top_num();
			if (tn < m_count) {
				LIBVSQ_ASSERT_I_("fetch", tn == m_count-1);
				m_queue.pop();
			}
		}
	}
	const char *fetch_()
	{
		if (m_queue.is_empty() || m_queue.top_num() > m_count)
			return NULL;
		m_count++;
		return m_queue.top_text();
	}
public:
	void reset(int i)
	{
		this->flush_top_();
		bool e = m_queue.is_empty();
		m_initial = m_count = i;
		m_queue.flush();
		if (!e)
			throw Ex::LackedDMFrame();
	}
	const char *operator () (const char *text)
	{
		this->flush_top_();

		if (text == NULL)
			return this->fetch_();

		int num;
		text = Impl_::check_vsq_text_(text, &num);
		if (text == NULL)
			throw Ex::NotDMText();
		if (m_count > num)
			throw Ex::DuplicatedDMFrame();
		if (m_count < num) {
			m_queue.push(num, text);
			return this->fetch_();
		}
		m_count++;
		return text;
	}
	Impl_(int i) : m_initial(i), m_count(i) { }
	~Impl_() { }
};

SortDMText::SortDMText(int i) : m_impl(new Impl_(i)) { }

SortDMText::~SortDMText() { }

void SortDMText::reset(int i) { m_impl->reset(i); }

const char *SortDMText::operator () (const char *text)
{ return (*m_impl)(text); }

} // LibVSQ
