#include <libvsq/Track.h>
#include <libvsq/Exception.h>
#include <libvsq/Mixer.h>
#include <libvsq/EventList.h>
#include <libvsq/EventDataDB.h>
#include <libvsq/EventHandle.h>
#include <libvsq/BPList.h>
#include <vector>

namespace LibVSQ
{

class Track::Impl_
{
	Tracks::Common m_common;
	Tracks::Master *m_master;
	Mixer *m_mixer;
	EventList m_events;
	EventDatas::Database m_event_datas;
	EventHandles::Database m_handles;
	BPLists::Database m_bplists;
public:
	Impl_() : m_master(NULL), m_mixer(NULL) { }
	~Impl_() { delete m_master; delete m_mixer; }
	bool is_master_valid() const { return m_master != NULL; }
	bool is_mixer_valid() const { return m_mixer != NULL; }
	AMT_ACSOR_RW(Tracks::Common, common);
	AMT_PTR_TO_REF_ACSOR_CONST(const Tracks::Master, master);
	AMT_ACSOR_W(Tracks::Master *, master);
	AMT_PTR_TO_REF_ACSOR_CONST(const Mixer, mixer);
	AMT_ACSOR_W(Mixer *, mixer);
	AMT_ACSOR_RW(EventList, events);
	AMT_ACSOR_RW(EventDatas::Database, event_datas);
	AMT_ACSOR_RW(EventHandles::Database, handles);
	AMT_ACSOR_RW(BPLists::Database, bplists);
};

Track::Track() : m_impl(new Impl_) { }

Track::~Track() { }

bool Track::is_master_valid() const { return m_impl->is_master_valid(); }

bool Track::is_mixer_valid() const { return m_impl->is_mixer_valid(); }

AMT_DEF_IMPL_ACSOR_RW(Track, Tracks::Common, common);

AMT_DEF_IMPL_ACSOR_R(Track, Tracks::Master, master);
AMT_DEF_IMPL_ACSOR_W(Track, Tracks::Master *, master);

AMT_DEF_IMPL_ACSOR_R(Track, Mixer, mixer);
AMT_DEF_IMPL_ACSOR_W(Track, Mixer *, mixer);

AMT_DEF_IMPL_ACSOR_RW(Track, EventList, events);

AMT_DEF_IMPL_ACSOR_W(Track, EventDatas::Database, event_datas);
AMT_DEF_IMPL_ACSOR_W(Track, EventHandles::Database, handles);

AMT_DEF_IMPL_ACSOR_RW(Track, BPLists::Database, bplists);


} // LibVSQ
