#ifndef LIBVSQ_LINE_BUFFER_H_
#define LIBVSQ_LINE_BUFFER_H_

#include <libvsq/Defines.h>
#include <libvsq/Types.h>
#include <libvsq/Assert.h>

namespace LibVSQ
{

class LineBuffer
{
	const char *m_buffer;
	Size m_length;
public:
	LineBuffer() : m_buffer(NULL), m_length(0) { }
	LineBuffer(const char *b, Size l) : m_buffer(l?b:NULL), m_length(b?l:0)
	{ }
	LineBuffer(const char *);
	void reset() { m_buffer = NULL; m_length = 0; }
	AMT_ACSOR_R(const char *, buffer);
	AMT_ACSOR_R(Size, length);
	char operator [] (Size ofs) const { return m_buffer[ofs]; }
	bool is_empty() const { return m_length == 0; }
	void chomp();
	LineBuffer skip_to(char);
	LineBuffer separate(char);
	LineBuffer cut_left(Size);
	LineBuffer cut_right(Size);
	template <typename F_>
	LineBuffer span(F_ f)
	{
		Size l=0;
		while (l < m_length && f(m_buffer[l]))
			l++;
		LineBuffer ret(m_buffer+l, m_length-l);
		m_length = l;
		return ret;
	}
	int compare(const LineBuffer &) const;
	int compare(const char *str) const
	{ return this->compare(LineBuffer(str)); }
	bool match_prefix(const LineBuffer &) const;
	bool match_prefix(const char *str) const
	{ return this->match_prefix(LineBuffer(str)); }
	bool match_suffix(const LineBuffer &) const;
	bool match_suffix(const char *str) const
	{ return this->match_suffix(LineBuffer(str)); }
	const char *begin() const { return m_buffer; }
	const char *end() const { return m_buffer+m_length; }
};

} // LibVSQ

#endif
