#ifndef LIBVSQ_MIXER_H_
#define LIBVSQ_MIXER_H_

#include <libvsq/Defines.h>
#include <libvsq/Types.h>
#include <libvsq/Assert.h>
#include <amt/ScopedPtr.h>

namespace LibVSQ
{

namespace Mixers
{

class Param
{
	int m_fader;
	int m_panpot;
	bool m_mute;
	bool m_solo;
public:
	Param() : m_fader(0), m_panpot(0), m_mute(false), m_solo(false) { }
	AMT_ACSOR_RW(int, fader);
	AMT_ACSOR_RW(int, panpot);
	AMT_ACSOR_RW(bool, mute);
	AMT_ACSOR_RW(bool, solo);
};

} // Mixers

class Mixer
{
	AMT_DECL_INNER_FRIEND_CLASS(Impl_);
public:
	typedef Mixers::Param Param;
private:
	AMT::Ptr::Scoped<Impl_> m_impl;
	//
	Param &at_(int) const;
public:
	Mixer();
	~Mixer();
	int num_tracks() const;
	AMT_DECL_ACSOR_RW(Param, master);
	AMT_DECL_ACSOR_RW(int, output_mode);
	void set_num_tracks(int);
	Param &operator [] (int idx)
	{
		LIBVSQ_ASSERT_U_("operator []",
				 idx >= 0 && idx < this->num_tracks());
		return this->at_(idx);
	}
	const Param &operator [] (int idx) const
	{
		LIBVSQ_ASSERT_U_("operator []",
				 idx >= 0 && idx < this->num_tracks());
		return this->at_(idx);
	}
};

} // LibVSQ

#endif
