#ifndef LIBVSQ_PARSE_VSQ_H_
#define LIBVSQ_PARSE_VSQ_H_

#include <libvsq/Defines.h>
#include <libvsq/Types.h>
#include <libvsq/Assert.h>
#include <amsmf/InputBuffer.h>
#include <amt/ScopedPtr.h>

namespace LibVSQ
{

namespace ParseVSQs
{

class Hooks
{
public:
	virtual ~Hooks() = 0;
	virtual void begin_vsq(int, ClockLength) = 0;
	virtual void end_vsq() = 0;
	virtual void begin_master_track() = 0;
	virtual void end_master_track() = 0;
	virtual void change_tempo(ClockLength, double) = 0;
	virtual void change_time_signature(ClockLength, int, int) = 0;
	virtual void begin_voice_track(int) = 0;
	virtual void end_voice_track(int) = 0;
	virtual void begin_section(int, const char *) = 0;
	virtual void end_section(int, const char *) = 0;
	virtual void field(int, const char *, const char *, const char *) = 0;
};
AMT_DEF_INLINE_DTOR(Hooks);

} // ParseVSQs

class ParseVSQ
{
	AMT_EMBED_IMPL;
public:
	typedef ParseVSQs::Hooks Hooks;
	typedef AMSMF::InputBuffer InputBuffer;
	ParseVSQ(Hooks &);
	~ParseVSQ();
	bool operator () (InputBuffer &);
	void done();
	void reset(Hooks &);
};

} // LibVSQ

#endif
