#ifndef LIBVSQ_PARSE_VSQ_TEXT_H_
#define LIBVSQ_PARSE_VSQ_TEXT_H_

#include <libvsq/Defines.h>
#include <libvsq/Types.h>
#include <libvsq/Assert.h>
#include <libvsq/LineBuffer.h>
#include <amt/ScopedPtr.h>

namespace LibVSQ
{

namespace TextParsers
{

class Hooks
{
public:
	virtual ~Hooks() = 0;
	virtual void begin_section(const char *) = 0;
	virtual void end_section(const char *) = 0;
	virtual void field(const char *, const char *, const char *) = 0;
};
AMT_DEF_INLINE_DTOR(Hooks);

} // Parsers

class TextParser
{
	AMT_EMBED_IMPL;
public:
	typedef TextParsers::Hooks Hooks;
private:
	void ensure_valid_()
	{ LIBVSQ_ASSERT_U_("ensure_valid_", m_impl.is_valid()); }
	bool parse_(const LineBuffer &);
public:
	TextParser(Hooks &);
	~TextParser();
	void reset(Hooks &);
	void done();
	bool parse(const LineBuffer &l)
	{ this->ensure_valid_(); return this->parse_(l); }
	bool parse(const char *str =NULL)
	{ return this->parse(LineBuffer(str)); }
	bool parse(const char *str, Size len)
	{ return this->parse(LineBuffer(str, len)); }
};

} // LibVSQ

#endif
