#ifndef LIBVSQ_READ_SECTION_H_
#define LIBVSQ_READ_SECTION_H_

#include <libvsq/Defines.h>
#include <libvsq/Types.h>
#include <libvsq/Assert.h>
#include <amt/ScopedPtr.h>

namespace LibVSQ
{

class Track;

class ReadSection
{
public:
	virtual ~ReadSection() = 0;
	virtual void operator () (const char *, const char *) = 0;
	virtual void check_complete() const = 0;
	typedef AMT::Ptr::Scoped<ReadSection> ScopedPtr;
};
AMT_DEF_INLINE_DTOR(ReadSection);

class ReadSectionFactory
{
public:
	virtual ~ReadSectionFactory() = 0;
	virtual ReadSection *operator () (Track &, const char *) const = 0;
};
AMT_DEF_INLINE_DTOR(ReadSectionFactory);

template <class Class_>
class ReadSectionFactoryTmpl : public ReadSectionFactory
{
	ReadSectionFactoryTmpl() { }
public:
	static const ReadSectionFactory &get()
	{ static ReadSectionFactoryTmpl self; return self; }
	~ReadSectionFactoryTmpl() { }
	ReadSection *operator () (Track &, const char *) const;
};

} // LibVSQ

#endif
