#ifndef LIBVSQ_READ_SECTION_TCC_
#define LIBVSQ_READ_SECTION_TCC_

#include <libvsq/ReadSection.h>
#include <libvsq/KeyDispatch.tcc>
#include <libvsq/Track.h>
#include <libvsq/Exception.h>

namespace LibVSQ
{

template <class Class_>
ReadSection *
ReadSectionFactoryTmpl<Class_>::operator () (Track &tr, const char *sect) const
{
	return new Class_(tr, sect);
}

#define LIBVSQ_RH_MAKE_MASK_OPE_(t)	(t) |
#define LIBVSQ_RH_MAKE_MASK(set)	(set(LIBVSQ_RH_MAKE_MASK_OPE_) 0)
#define LIBVSQ_RH_DEF(tag, name)					      \
template <> RH_HANDLER(tag)::Key RH_HANDLER(tag)::key() { return name; }      \
template <> void RH_HANDLER(tag)::hook(Context &ctx, Arg a) const
#define LIBVSQ_RH_REGISTER_OPE_(t)	d.insert(RH_HANDLER(t)::handler());
#define LIBVSQ_RH_REGISTER(ctx, set)					      \
template <>								      \
void ReaderTmpl<ctx>::register_handlers(Dispatcher &d)			      \
{									      \
	set(LIBVSQ_RH_REGISTER_OPE_);					      \
}

template <class Ctx_, const typename Ctx_::Tag TAG_>
class HandlerTmpl : public KeyDispatchers::Handler<Ctx_>
{
public:
	typedef Ctx_ Context;
	typedef KeyDispatchers::Handler<Ctx_> Handler;
	typedef typename Ctx_::Tag Tag;
	typedef typename Ctx_::Key Key;
	typedef typename Ctx_::Arg Arg;
private:
	static Key key();
	HandlerTmpl() : Handler(HandlerTmpl::key()) { }
	void hook(Context &, Arg) const;
public:
	static const Handler &handler() { static HandlerTmpl h; return h; }
	void operator () (Context &ctx, Key, Arg a) const
	{
		if (ctx.is_set(TAG_, a))
			throw Ex::DuplicatedEntry();
		this->hook(ctx, a);
		ctx.mark_set(TAG_, a);
	}
};

template <class Ctx_>
class ReaderTmpl : public ReadSection
{
public:
	typedef Ctx_ Context;
	typedef KeyDispatchers::Handler<Ctx_> Handler;
	typedef KeyDispatcher<Handler> Dispatcher;
	typedef typename Ctx_::Tag Tag;
	typedef typename Ctx_::Key Key;
	typedef typename Ctx_::Arg Arg;
	typedef typename Ctx_::Data Data;
protected:
	static void register_handlers(Dispatcher &);  // must be defined.
	class DispatcherSingleton_;
	friend class DispatcherSingleton_;
	class DispatcherSingleton_
	{
		Dispatcher m_dispatcher;
	public:
		DispatcherSingleton_()
		{ ReaderTmpl::register_handlers(m_dispatcher); }
		AMT_ACSOR_RW(Dispatcher, dispatcher);
	};
	static const Dispatcher &dispatcher_()
	{
		static DispatcherSingleton_ d;
		return d.dispatcher();
	}
	//
	Ctx_ m_ctx;
public:
	ReaderTmpl(Track &tr, Data &d) : m_ctx(tr, d) { }
	~ReaderTmpl() { }
	void operator () (Key key, Arg val)
	{ ReaderTmpl::dispatcher_()(m_ctx, key, val); }
	void check_complete() const { m_ctx.check_complete(); }
};

template <class Data_, typename Tag_, const U32 MANDATORY_>
class SimpleContextTmpl
{
public:
	typedef Data_ Data;
	typedef Tag_ Tag;
	typedef const char *Key;
	typedef const char *Arg;
private:
	Track &m_track;
	Data &m_data;
	U32 m_mask;
public:
	SimpleContextTmpl(Track &tr, Data &d)
		: m_track(tr), m_data(d), m_mask(0) { }
	AMT_REF_ACSOR_CONST(Track, track);
	AMT_REF_ACSOR_CONST(Data, data);
	AMT_ACSOR_R(U32, mask);
	void check_complete_mandatory() const
	{
		if ((m_mask & MANDATORY_) != MANDATORY_)
			throw Ex::LackedEntry();
	}
	bool is_set(Tag t, Arg) const
	{
		return (m_mask & (U32)t) != 0;
	}
	void mark_set(Tag t, Arg)
	{
		m_mask |= (U32)t;
	}
	// need to define check_complete().
};

} // LibVSQ

#endif
