#ifndef LIBVSQ_SONG_H_
#define LIBVSQ_SONG_H_

#include <libvsq/Defines.h>
#include <libvsq/Types.h>
#include <libvsq/Assert.h>
#include <amt/ScopedPtr.h>

namespace LibVSQ
{

class Track;
class TempoMap;
class TimeSignMap;
namespace Tracks { class Master; }
class Mixer;

class Song
{
	AMT_NOCOPY(Song);
	AMT_EMBED_IMPL;
private:
	const Tracks::Master &master_() const;
	const Mixer &mixer_() const;
	Track &track_(int) const;
	Track &track__(int tr) const
	{
		LIBVSQ_ASSERT_U_("track", tr >= 0 && tr < this->num_tracks());
		return this->track_(tr);
	}
public:
	Song();
	~Song();
	AMT_DECL_ACSOR_RW(ClockLength, division);
	bool is_master_valid() const;
	const Tracks::Master &master() const
	{
		LIBVSQ_ASSERT_U_("master", this->is_master_valid());
		return this->master_();
	}
	bool is_mixer_valid() const;
	const Mixer &mixer() const
	{
		LIBVSQ_ASSERT_U_("mixer", this->is_mixer_valid());
		return this->mixer_();
	}
	int num_tracks() const;
	AMT_DECL_ACSOR_RW(TempoMap, tempo_map);
	AMT_DECL_ACSOR_RW(TimeSignMap, time_sign_map);
	Track &insert_track();
	Track &track(int tr) { return this->track__(tr); }
	const Track &track(int tr) const { return this->track__(tr); }
};

} // LibVSQ

#endif
