#ifndef LIBVSQ_TRACK_H_
#define LIBVSQ_TRACK_H_

#include <libvsq/Defines.h>
#include <libvsq/Types.h>
#include <libvsq/Assert.h>
#include <amt/ScopedPtr.h>
#include <string>

namespace LibVSQ
{

class EventList;
namespace EventHandles { class Database; }
namespace EventDatas { class Database; }
namespace BPLists { class Database; }
class Mixer;

namespace Tracks
{

class Common
{
	std::string m_version;
	std::string m_name;
	RGB888 m_color;
	int m_dynamics_mode;
	int m_play_mode;
public:
	Common() { }
	AMT_ACSOR_RW(std::string, version);
	AMT_ACSOR_RW(std::string, name);
	AMT_ACSOR_RW(RGB888, color);
	AMT_ACSOR_RW(int, dynamics_mode);
	AMT_ACSOR_RW(int, play_mode);
};

class Master
{
	int m_pre_measure;
public:
	Master() { }
	AMT_ACSOR_RW(int, pre_measure);
};

} // Tracks

class Track
{
	AMT_NOCOPY(Track);
	AMT_EMBED_IMPL;
public:
	Track();
	~Track();
	AMT_DECL_ACSOR_RW(Tracks::Common, common);
	bool is_master_valid() const;
	AMT_DECL_ACSOR_R(Tracks::Master, master);
	AMT_DECL_ACSOR_W(Tracks::Master *, master);
	bool is_mixer_valid() const;
	AMT_DECL_ACSOR_R(Mixer, mixer);
	AMT_DECL_ACSOR_W(Mixer *, mixer);
	AMT_DECL_ACSOR_RW(EventList, events);
	AMT_DECL_ACSOR_W(EventDatas::Database, event_datas);
	AMT_DECL_ACSOR_W(EventHandles::Database, handles);
	AMT_DECL_ACSOR_RW(BPLists::Database, bplists);
};

} // LibVSQ

#endif
