#include <libvsq/Track.h>
#include <libvsq/EventDataDB.h>
#include <libvsq/EventList.h>
#include <libvsq/EventData.h>
#include <libvsq/EventHandle.h>
#include <libvsq/ReadSectEventList.h>
#include <libvsq/ReadSectID.h>
#include <libvsq/ReadSectHandle.h>

void
print(const LibVSQ::EventHandles::Singer &sh)
{
	printf("      icon id: %s\n", sh.icon_id().c_str());
	printf("      ids: %s\n", sh.ids().c_str());
	printf("      original: %d\n", sh.original());
	printf("      caption: %s\n", sh.caption().c_str());
	printf("      length: %u\n", (unsigned)sh.length());
	printf("      lang: %d\n", sh.language());
	printf("      prog: %d\n", sh.program());
}

void
print(const LibVSQ::EventHandles::Lyric &lh)
{
	int nl = lh.num_lyrics();
	for (int i=0; i<nl; i++) {
		const LibVSQ::EventHandles::LyricField &lf = lh[i];
		printf("      %s:\n", lf.key().c_str());
		printf("        lyrics: %s\n", lf.lyric().c_str());
		printf("        phoneme: %s\n", lf.phoneme().c_str());
		printf("        param1: %f\n", lf.param1());
		printf("        param2: %d\n", lf.param2());
		printf("        param3: %d\n", lf.param3());
	}
}

void
print(const LibVSQ::EventHandles::VibratoChanges &ch)
{
	printf("        start: %d\n", (int)ch.start_value());
	int n = (int)ch.num_changes();
	printf("        num changes: %d\n", n);
	for (int i=0; i<n; i++) {
		const LibVSQ::EventHandles::VibratoCoord &co = ch[i];
		printf("        ch[%d]: (%f, %u)\n", i, co.pos(), co.value());
	}
}

void
print(const LibVSQ::EventHandles::Vibrato &vh)
{
	printf("      icon id: %s\n", vh.icon_id().c_str());
	printf("      ids: %s\n", vh.ids().c_str());
	printf("      original: %d\n", vh.original());
	printf("      caption: %s\n", vh.caption().c_str());
	printf("      length: %u\n", (unsigned)vh.length());
	printf("      depth:\n");
	print(vh.depth());
	printf("      rate:\n");
	print(vh.rate());
}

int
main()
{
	LibVSQ::Track tr;
	LibVSQ::ReadSection::ScopedPtr rs;

	rs.reset(LibVSQ::ReadSectEventListFactory::get()(tr, "EventList"));
	(*rs)("0", "ID#0000");
	(*rs)("960", "ID#0001");
	(*rs)("4800", "EOS");
	rs.reset();

	rs.reset(LibVSQ::ReadSectIDFactory::get()(tr, "ID#0000"));
	(*rs)("Type", "Singer");
	(*rs)("IconHandle", "h#0000");
	rs.reset(LibVSQ::ReadSectIDFactory::get()(tr, "ID#0001"));
	(*rs)("Type", "Anote");
	(*rs)("Length", "120");
	(*rs)("Note#", "60");
	(*rs)("Dynamics", "64");
	(*rs)("PMBendDepth", "20");
	(*rs)("PMBendLength", "20");
	(*rs)("PMbPortamentoUse", "0");
	(*rs)("DEMdecGainRate", "50");
	(*rs)("DEMaccent", "50");
	(*rs)("LyricHandle", "h#0001");
	(*rs)("VibratoHandle", "h#0002");
	(*rs)("VibratoDelay", "60");
	rs.reset();

	rs.reset(LibVSQ::ReadSectHandleFactory::get()(tr, "h#0000"));
	(*rs)("IconID", "$07010000");
	(*rs)("IDS", "Miku");
	(*rs)("Original", "0");
	(*rs)("Caption", "");
	(*rs)("Length", "1");
	(*rs)("Language", "0");
	(*rs)("Program", "0");
	rs.reset(LibVSQ::ReadSectHandleFactory::get()(tr, "h#0001"));
	(*rs)("L0", "\"\",\"a\",1.000000,0,0");
	rs.reset(LibVSQ::ReadSectHandleFactory::get()(tr, "h#0002"));
	(*rs)("IconID", "$04040001");
	(*rs)("IDS", "normal");
	(*rs)("Original", "1");
	(*rs)("Caption", "");
	(*rs)("Length", "60");
	(*rs)("StartDepth", "64");
	(*rs)("DepthBPNum", "3");
	(*rs)("DepthBPX", "0.200000,0.300000,0.400000");
	(*rs)("DepthBPY", "70,80,90");
	(*rs)("StartRate", "64");
	(*rs)("RateBPNum", "4");
	(*rs)("RateBPX", "0.300000,0.400000,0.500000,0.600000");
	(*rs)("RateBPY", "70,80,90,100");
	rs.reset();

	printf("result:\n");
	LibVSQ::EventList &l = tr.events();
	int nev = l.num_entries();
	for (int i=0; i<nev; i++) {
		printf("  %d: %s\n",
		       (int)l[i].pos(), l[i].entry()->name().c_str());
		switch (l[i].entry()->data().type()) {
		case LibVSQ::EventData::T_SINGER:
		{
			const LibVSQ::EventDatas::Singer &s =
			    l[i].entry()->data().to<
			    LibVSQ::EventDatas::Singer>();
			printf("    Type: Singer\n");
			printf("    IconHandle: %s\n",
			       s.icon_handle().name().c_str());
			const LibVSQ::EventHandles::Singer &sh =
			    s.icon_handle().to<LibVSQ::EventHandles::Singer>();
			print(sh);
			break;
		}
		case LibVSQ::EventData::T_ANOTE:
		{
			const LibVSQ::EventDatas::ANote &n =
			    l[i].entry()->data().to<
			    LibVSQ::EventDatas::ANote>();
			printf("    Type: Anote\n");
			printf("    Length: %u\n", (unsigned)n.length());
			printf("    Note#: %u\n", (unsigned)n.note_no());
			printf("    Dynamics: %u\n", (unsigned)n.dynamics());
			printf("    PMBendDepth: %u\n",
			       (unsigned)n.pm_bend_depth());
			printf("    PMBendLength: %u\n",
			       (unsigned)n.pm_bend_length());
			printf("    PMbPortamentoUse: %u\n",
			       (unsigned)n.portament_use());
			printf("    DEMdecGainRate: %u\n",
			       (unsigned)n.decay_rate());
			printf("    DEMaccent: %u\n", (unsigned)n.accent());
			printf("    LyricHandle: %s\n",
			       n.lyric_handle().name().c_str());
			const LibVSQ::EventHandles::Lyric &lh =
			    n.lyric_handle().to<LibVSQ::EventHandles::Lyric>();
			print(lh);
			if (n.is_vibrato_valid()) {
				printf("    VibratoHandle: %s\n",
				       n.vibrato_handle().name().c_str());
				const LibVSQ::EventHandles::Vibrato &vh =
				    n.vibrato_handle().to<
				    LibVSQ::EventHandles::Vibrato>();
				print(vh);
				printf("    VibratoDelay: %u\n",
				       (unsigned)n.vibrato_delay());
			} else
				printf("    no Vibrato\n");
			break;
		}
		case LibVSQ::EventData::T_EOS:
			break;
		}
	}

	return 0;
}
