#include <libvsq/Track.h>
#include <libvsq/EventDataDB.h>
#include <libvsq/EventList.h>
#include <libvsq/EventData.h>
#include <libvsq/EventHandle.h>
#include <libvsq/ReadSectEventList.h>
#include <libvsq/ReadSectID.h>

int
main()
{
	LibVSQ::Track tr;
	LibVSQ::ReadSection::ScopedPtr rsel;
	LibVSQ::ReadSection::ScopedPtr rsid;

	rsel.reset(LibVSQ::ReadSectEventListFactory::get()(tr, "EventList"));
	(*rsel)("0", "ID#0000");
	(*rsel)("960", "ID#0001");
	(*rsel)("4800", "EOS");
	rsel.reset();

	rsid.reset(LibVSQ::ReadSectIDFactory::get()(tr, "ID#0000"));
	(*rsid)("Type", "Singer");
	(*rsid)("IconHandle", "h#0000");
	rsid.reset(LibVSQ::ReadSectIDFactory::get()(tr, "ID#0001"));
	(*rsid)("Type", "Anote");
	(*rsid)("Length", "120");
	(*rsid)("Note#", "60");
	(*rsid)("Dynamics", "64");
	(*rsid)("PMBendDepth", "20");
	(*rsid)("PMBendLength", "20");
	(*rsid)("PMbPortamentoUse", "0");
	(*rsid)("DEMdecGainRate", "50");
	(*rsid)("DEMaccent", "50");
	(*rsid)("LyricHandle", "h#0001");
	(*rsid)("VibratoHandle", "h#0002");
	(*rsid)("VibratoDelay", "60");
	rsid.reset();

	printf("result:\n");
	LibVSQ::EventList &l = tr.events();
	int nev = l.num_entries();
	for (int i=0; i<nev; i++) {
		printf("  %d: %s\n",
		       (int)l[i].pos(), l[i].entry()->name().c_str());
		switch (l[i].entry()->data().type()) {
		case LibVSQ::EventData::T_SINGER:
		{
			const LibVSQ::EventDatas::Singer &s =
			    l[i].entry()->data().to<
			    LibVSQ::EventDatas::Singer>();
			printf("    Type: Singer\n");
			printf("    IconHandle: %s\n",
			       s.icon_handle().name().c_str());
			break;
		}
		case LibVSQ::EventData::T_ANOTE:
		{
			const LibVSQ::EventDatas::ANote &n =
			    l[i].entry()->data().to<
			    LibVSQ::EventDatas::ANote>();
			printf("    Type: Anote\n");
			printf("    Length: %u\n", (unsigned)n.length());
			printf("    Note#: %u\n", (unsigned)n.note_no());
			printf("    Dynamics: %u\n", (unsigned)n.dynamics());
			printf("    PMBendDepth: %u\n",
			       (unsigned)n.pm_bend_depth());
			printf("    PMBendLength: %u\n",
			       (unsigned)n.pm_bend_length());
			printf("    PMbPortamentoUse: %u\n",
			       (unsigned)n.portament_use());
			printf("    DEMdecGainRate: %u\n",
			       (unsigned)n.decay_rate());
			printf("    DEMaccent: %u\n", (unsigned)n.accent());
			printf("    LyricHandle: %s\n",
			       n.lyric_handle().name().c_str());
			if (n.is_vibrato_valid()) {
				printf("    VibratoHandle: %s\n",
				       n.vibrato_handle().name().c_str());
				printf("    VibratoDelay: %u\n",
				       (unsigned)n.vibrato_delay());
			} else
				printf("    no Vibrato\n");
			break;
		}
		case LibVSQ::EventData::T_EOS:
			break;
		}
	}

	return 0;
}
